//**************************************************
// Copyright (c) 2015 Cisco Systems, Inc.
// All rights reserved.
//**************************************************
package com.cisco.ise.ers.demo;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.security.Security;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBElement;

import com.cisco.ise.ers.ca.ERSEndPointCert;
import com.cisco.ise.ers.ca.ERSEndPointCert.CertificateRequest;
import com.cisco.ise.ers.ca.ERSEndPointCert.CertificateRequest.Entry;
import com.cisco.ise.ers.ca.ObjectFactory;

import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPut;
import org.bouncycastle.jcajce.provider.BouncyCastleFipsProvider;

public class EndPointCertDemo {
    
    private static final String CERT_TEMPLATE_NAME  = "EAP_Authentication_Certificate_Template";
    private static final String DESCRIPTION         = "ERS EndPointCert demo";
    private static final String CERT_FORMAT         = "PKCS8";
    private static final String PASSWORD            = "Demo1234";
    
    private static ObjectFactory caFactory = new ObjectFactory();
    private static JAXBHelper jaxbHelper = new JAXBHelper(caFactory.getClass().getPackage().getName());

    public static void main(String[] args) {
        String serverHost = null;
        String userName = null;
        String password = null;

        if (args != null && args.length == 3) {
            
            serverHost = args[0]; // Server Host
            userName = args[1]; // User name
            password = args[2]; // User password
            
            println("\n ISE ip:" + serverHost + " userName:" + userName + " psw:" + password);
            
            // Get an instance of Rest Client
            ERSClient ersClient = new ERSClient(userName, password, serverHost);
            
            println("\n\n\n\n\n\n" + "--------------------------------- START --------------------------------------------------");
            
            // Connectivity Check
            connectivityCheck(ersClient);
            
            // Get all endpoints
            println("\n\n\n\n\n\n" + "----------- Create Certificate ----------------------------------------------------------------");
            println("\n###############################################################" + 
                    "\n#  Create Certificate:                                        #" +
                    "\n#  This demo sends PUT request to server to create an         #" +
                    "\n#  EndPoint certificate. The expected response will be status #" +
                    "\n#  200 with a ZIP file in the response body. This ZIP file    #" +
                    "\n#  contain a PEM Certificate file and a PEM PKCS8 private key #" +
                    "\n#  file.                                                      #" +
                    "\n###############################################################");
            
            createCert(ersClient, userName);

        } else {
			println("\n illegal argument. \n syntax: \n EndPointCertDemo.sh [host-ip] [user-name] [password]\n");
        }
    }
    
    private static void createCert(ERSClient ersClient, String userName) {
        println("\n\n------------ FYI ---------------------");
        println("\n# ISE needs to have the default certificate template with the name: " + CERT_TEMPLATE_NAME + 
                "\n# in order for this demo to function properly. This Cert Template needs to have a reference to" + 
                "\n# the Internal CA SCEP profile.\n");

        HttpResponse response = null;
        ZipInputStream zipStream = null;
        try {
            // EP Cert object as a request
            ERSEndPointCert epCertReq = new ERSEndPointCert();

            // Request data
            CertificateRequest certReq = caFactory.createERSEndPointCertCertificateRequest();
            List<Entry> entries = certReq.getEntry();

            // Populate the request data
            Entry cnData = new Entry();
            cnData.setKey("cn");
            cnData.setValue(userName);
            entries.add(cnData);
            Entry sanData = new Entry();
            sanData.setKey("san");
            sanData.setValue("11-22-33-44-55-66");
            entries.add(sanData);

            epCertReq.setCertificateRequest(certReq);
            epCertReq.setCertTemplateName(CERT_TEMPLATE_NAME);
            epCertReq.setFormat(CERT_FORMAT);
            epCertReq.setPassword(PASSWORD);
            
            JAXBElement<ERSEndPointCert> jaxbEndPointCert = caFactory.createEndpointcert(epCertReq);
            String epCertAsXml = jaxbHelper.marshal(jaxbEndPointCert);

            println("REQUEST PAYLOAD:\n");
            println(epCertAsXml);
            
            response = ersClient.customOperation("ca", "endpointcert", epCertAsXml, "certRequest", HttpPut.METHOD_NAME);
            if (response == null) {
                println("###############################################");
                println("# Internal error, unable to get response data #");
                println("###############################################");
                return;
            }
            zipStream = new ZipInputStream(response.getEntity().getContent());
            
            ZipEntry entry = zipStream.getNextEntry();
            while (entry != null) {
                println("###########################################################################");
                println("# File found in ZIP: " + entry.getName());
                println("###########################################################################");
                if (entry.getName().endsWith(".cer")) {
                    println("##################################");
                    println("# Found Certificate Entry in Zip #");
                    println("##################################");
                    Security.addProvider(new BouncyCastleFipsProvider());
                    CertificateFactory certGen = CertificateFactory.getInstance("X.509", "BCFIPS");
                    X509Certificate epCert = (X509Certificate)certGen.generateCertificate(zipStream);
                    println(epCert.toString());
                }
                entry = zipStream.getNextEntry();
            }
        } catch (Exception e) {
            e.printStackTrace(); // NOPMD
        }finally{
            if (zipStream != null) {
                try { zipStream.close(); } catch (IOException e) { e.printStackTrace(); } // NOPMD
            }
            //release the connection
            if(response != null){
                try {
                    if (response.getEntity() != null) {
                        response.getEntity().consumeContent();
                    }
                } catch (IOException e) {
                    e.printStackTrace(); // NOPMD
                }
            }
        }
    }
        
    /**
     * connectivityCheck
     * send getVersion request to server in order to try connect
     * and make sure the credentials are OK
     * @param ersClient
     */
    private static void connectivityCheck(ERSClient ersClient) {
        HttpResponse response = null;
        try {
            
            println("\n###############################################################" + 
                               "\n#  Connectivity Check:                                        #" +
                               "\n#  About to send GET Endpoint Cert version request to server  #" +
                               "\n#  in order to make sure that connection can be established.  #" +
                               "\n###############################################################");
            
            //getting endpoint server version, just for checking that the  
            response = ersClient.getVersion("ca","endpointcert");
            String responseAsString = RestUtils.getResponseContentAsString(response);
            int responseStatus = RestUtils.getResponseStatus(response);
            if (responseStatus == HttpServletResponse.SC_OK) {
                println("\nConnection to ISE server success");
            }else if(responseStatus == HttpServletResponse.SC_UNAUTHORIZED){
                println("\n###############################################################" + 
                                   "\n#  401 - UNAUTHORIZED:                                        #" +
                                   "\n#  Connection to ISE server failed !!!                        #" +
                                   "\n#  Make sure you are using  valid Admin Credentials.          #" +
                                   "\n#  Please note that this demo works for ERS Admins, Internal  #" +
                                   "\n#  users, and AD/LDAP users                                   #" +
                                   "\n###############################################################");                
                System.exit(-1);
            }else{
                println("\nRESPONSE CONTENT: \n" + " " +  responseAsString);
            }
        } catch (Exception e) {
            e.printStackTrace(); // NOPMD
        } finally{
            //release the connection
            if(response != null){
                try {
                    if (response.getEntity() != null) {
                        response.getEntity().consumeContent();
                    }
                } catch (IOException e) {
                    e.printStackTrace(); // NOPMD
                }
            }
        }
    }

    private static void println(String string) {
        System.out.println(string); // NOPMD
    }
}
